@@ -13,9 +13,10 @@ fn main() {
     let mut cflags;
 
     if let Ok(commands_json) = env::var("RIOT_COMPILE_COMMANDS_JSON") {
+        println!("cargo:rerun-if-env-changed=RIOT_COMPILE_COMMANDS_JSON");
         println!("cargo:rerun-if-changed={}", commands_json);
-        let commands_file = std::fs::File::open(commands_json)
-            .expect("Failed to open RIOT_COMPILE_COMMANDS_JSON");
+        let commands_file =
+            std::fs::File::open(commands_json).expect("Failed to open RIOT_COMPILE_COMMANDS_JSON");
 
         #[derive(Debug, serde::Deserialize)]
         struct Entry {
@@ -28,25 +29,34 @@ fn main() {
         let any = &parsed[0];
 
         cc = any.arguments[0].clone();
-        cflags = shlex::join(any.arguments[1..]
-                             .iter()
-                             .map(|s| s.as_str())
-                             // Anything after -c is not CFLAGS but concrete input/output stuff
-                             .take_while(|&s| s != "-c")
-                             );
+        cflags = shlex::join(
+            any.arguments[1..]
+                .iter()
+                .map(|s| s.as_str())
+                // Anything after -c is not CFLAGS but concrete input/output stuff
+                .take_while(|&s| s != "-c"),
+        );
 
+        println!("cargo:rerun-if-env-changed=RIOT_USEMODULE");
         let usemodule = env::var("RIOT_USEMODULE")
             .expect("RIOT_USEMODULE is required when RIOT_COMPILE_COMMANDS_JSON is given");
         for m in usemodule.split(" ") {
             // Hack around https://github.com/RIOT-OS/RIOT/pull/16129#issuecomment-805810090
-            write!(cflags, " -DMODULE_{}", m.to_uppercase()).unwrap();
+            write!(
+                cflags,
+                " -DMODULE_{}",
+                m.to_uppercase()
+                    // avoid producing MODULE_BOARDS_COMMON_SAMDX1-ARDUINO-BOOTLOADER
+                    .replace('-', "_")
+            )
+            .unwrap();
         }
     } else {
         cc = env::var("RIOT_CC")
             .expect("Please pass in RIOT_CC; see README.md for details.")
             .clone();
-        cflags =
-            env::var("RIOT_CFLAGS").expect("Please pass in RIOT_CFLAGS; see README.md for details.");
+        cflags = env::var("RIOT_CFLAGS")
+            .expect("Please pass in RIOT_CFLAGS; see README.md for details.");
     }
 
     println!("cargo:rerun-if-env-changed=RIOT_CC");
@@ -141,7 +151,7 @@ fn main() {
             r"
 
 #ifdef {macro_name}
-static {type_name} init_{macro_name}(void) {{
+{type_name} init_{macro_name}(void) {{
     {type_name} result = {macro_name};
     return result;
 }}
@@ -208,11 +218,6 @@ static {type_name} init_{macro_name}(void) {{
     }
 
     let output = out_path.join(c2rust_outfile);
-    match std::fs::remove_file(&output) {
-        Ok(_) => (),
-        Err(e) if e.kind() == std::io::ErrorKind::NotFound => (),
-        Err(e) => panic!("Failed to remove output file: {}", e),
-    }
 
     let arguments: Vec<_> = core::iter::once("any-cc".to_string())
         .chain(cflags.into_iter())
@@ -247,6 +252,8 @@ static {type_name} init_{macro_name}(void) {{
             "--emit-modules",
             "--emit-no-std",
             "--translate-const-macros",
+            "--overwrite-existing",
+            "--fail-on-error",
         ])
         .status()
         .expect("C2Rust failed");
@@ -270,46 +277,67 @@ static {type_name} init_{macro_name}(void) {{
         .expect("Failed to read from riot_c2rust.rs");
 
     rustcode = rustcode.replace("use ::libc;\n", "");
-    rustcode = rustcode.replace(r#"unsafe extern "C" fn "#, r#"pub unsafe fn "#);
-    // This only matches when c2rust is built to even export body-less functions
-    rustcode = rustcode.replace("    #[no_mangle]\n    fn ", "    #[no_mangle]\n    pub fn ");
-    // used as a callback, therefore does need the extern "C" -- FIXME probably worth a RIOT issue
-    rustcode = rustcode.replace(
-        r"pub unsafe fn _evtimer_msg_handler",
-        r#"pub unsafe extern "C" fn _evtimer_msg_handler"#,
-    );
-    rustcode = rustcode.replace(
-        r"pub unsafe fn _evtimer_mbox_handler",
-        r#"pub unsafe extern "C" fn _evtimer_mbox_handler"#,
-    );
-    // same problem but from C2Rust's --translate-const-macros
-    rustcode = rustcode.replace(
-        r"pub unsafe fn __NVIC_SetPriority",
-        r#"pub unsafe extern "C" fn __NVIC_SetPriority"#,
-    );
+
     // C2Rust still generates old-style ASM -- workaround for https://github.com/immunant/c2rust/issues/306
     rustcode = rustcode.replace(" asm!(", " llvm_asm!(");
-    // particular functions known to be const because they have macro equivalents as well
-    // (Probably we could remove the 'extern "C"' from all functions)
-    rustcode = rustcode.replace(
-        r#"pub unsafe extern "C" fn mutex_init("#,
-        r#"pub const unsafe fn mutex_init("#,
+
+    // This only matches when c2rust is built to even export body-less functions
+    rustcode = rustcode.replace("    #[no_mangle]\n    fn ", "    #[no_mangle]\n    pub fn ");
+
+    // The GCLK global constant interferes with arguments and fields named GCLK in bindgen, as
+    // these would (due to the `use inline::*;` blanket import) try to make that into a matchable
+    // pattern. Renaming the whole constant.
+    //
+    // This is needed, for example, on the adafruit-itsybitsy-m4 board with the riot-hello-world
+    // example.
+    let gclk_word = regex::Regex::new(r"\bGCLK\b").expect("Invalid static regular expression");
+    rustcode = gclk_word.replace_all(&rustcode, "GLOBAL_GCLK").to_string();
+
+    // Replace the function declarations with ... usually something pub, but special considerations
+    // may apply
+    let mut rustcode_functionsreplaced = String::new();
+    let function_original_prefix = r#"unsafe extern "C" fn "#;
+    let mut functionchunks = rustcode.split(function_original_prefix);
+    rustcode_functionsreplaced.push_str(
+        functionchunks
+            .next()
+            .expect("Split produces at least a hit"),
     );
 
-    for (macro_name, _) in struct_initializers.iter() {
-        let search = format!(r#"pub unsafe fn init_{}"#, macro_name);
-        let replace = format!(r#"pub const fn init_{}"#, macro_name);
-        rustcode = rustcode.replace(&search, &replace);
+    for chunk in functionchunks {
+        let funcname = &chunk[..chunk.find('(').expect("Function has parentheses somewhere")];
+        let new_prefix = match funcname {
+            // used as a callback, therefore does need the extern "C" -- FIXME probably worth a RIOT issue
+            "_evtimer_msg_handler" | "_evtimer_mbox_handler" => function_original_prefix,
+
+            // Assigned by CMSIS to some const; see also riot-c2rust.h
+            "__masked_builtin_arm_get_fpscr" | "__masked_builtin_arm_set_fpscr" => {
+                function_original_prefix
+            }
+
+            // same problem but from C2Rust's --translate-const-macros
+            "__NVIC_SetPriority" => function_original_prefix,
+
+            // As below (no need for extern), and they are const by construction.
+            s if s.len() > 5
+                && &s[..5] == "init_"
+                && struct_initializers
+                    .iter()
+                    .any(|(macro_name, _)| &s[5..] == *macro_name) =>
+            {
+                // No "pub" because that's already a "pub" in front of it, they were never static
+                "const fn "
+            }
+
+            // The rest we don't need to call through the extern convention, but let's please make
+            // them pub to be usable
+            _ => "pub unsafe fn ",
+        };
+        rustcode_functionsreplaced.push_str(new_prefix);
+        rustcode_functionsreplaced.push_str(chunk);
     }
 
-    // Workaround for https://github.com/immunant/c2rust/issues/345
-    //
-    // As these are not really in the call tree of any public RIOT function, panicing probably good
-    // enough.
-    rustcode = rustcode.replace("__builtin_arm_get_fpscr()",
-            "panic!(\"fpscr could not be translated\")");
-    rustcode = rustcode.replace("__builtin_arm_set_fpscr(fpscr)",
-            "panic!(\"fpscr could not be translated\")");
+    rustcode = rustcode_functionsreplaced;
 
     let output_replaced = out_path.join("riot_c2rust_replaced.rs");
     std::fs::File::create(output_replaced)
